#include "tmp/libpq-fs.h"
#include  <sys/file.h>

void main(argc,argv)
     int argc;       
     char *argv[];
{
    int fd;
    int cnt, total = 0;
    char *buf;
    int blen;
    extern char *getenv();

    PQsetdb(getenv("USER"));
    printf ("p_rename(%s,%s) returns %d\n",argv[1],argv[2],
	    p_rename(argv[1],argv[2]));
    PQfinish();
}
