#include "tmp/libpq-fs.h"
extern char cwdir[1024];
void main ()
{
    char s[1024];

    printf ("cwd: %s\n",cwdir);
    printf ("/a = %s\n",resolve_path("/a"));
    printf ("/a = %s\n",resolve_path("a"));
    printf ("/a = %s\n",resolve_path("../a"));
    strcpy(cwdir,"/snake/eleven/craps");
    printf ("/snake/eleven = %s\n",resolve_path(".."));
    printf ("/snake = %s\n",resolve_path("../../././"));
    printf ("/snake/eyes = %s\n",resolve_path("../../././eyes"));
}
